@@ -1,3 +1,63 @@
+#' @title Transform a StMAR model parameter vector to a corresponding G-StMAR model parameter vector
+#'  with large dfs parameters reduced.
+#'
+#' @description \code{stmarpars_to_gstmar} transforms a StMAR model parameter vector to a corresponding
+#'  G-StMAR model parameter vector with large dfs parameters reduced by turning the related regimes GMAR type.
+#'
+#' @inheritParams loglikelihood_int
+#' @param maxdf regimes with degrees of freedom parameter value large than this will be turned into
+#'  GMAR type.
+#' @return Returns a list with three elements: \code{$params} contains the corresponding G-StMAR model
+#'  parameter vector, \code{$reg_order} contains the permutation that was applied to the regimes
+#'  (GMAR type components first, and decreasing ordering by mixign weight parameters), and
+#'  \code{$M} a vector of length two containing the number of GMAR type regimes in the first element
+#'  and the number of StMAR type components in the second.
+#' @examples
+#'  params12 <- c(2, 0.9, 0.1, 0.8, 0.5, 0.5, 0.4, 12, 300)
+#'  stmarpars_to_gstmar(1, 2, params12, maxdf=100)
+#' @export
+
+stmarpars_to_gstmar <- function(p, M, params, restricted=FALSE, constraints=NULL, maxdf=100) {
+  checkPM(p, M, model="StMAR")
+  check_params_length(p, M, params, model="StMAR", restricted=restricted, constraints=constraints)
+  checkConstraintMat(p, M, restricted=restricted, constraints=constraints)
+
+  dfs <- pick_dfs(p, M, params, model="StMAR")
+  if(!any(dfs > maxdf)) {
+    warning("No degrees of freedom parameter is larger than 'maxdf'. The original model is returned.")
+    return(list(params=params, reg_order=1:M, M=c(0, M)))
+  }
+  regs_to_change <- which(dfs > maxdf)
+  if(length(regs_to_change) == M) message("All regimes are changed to GMAR type. Thus the result is a GMAR model and not a G-StMAR model.")
+  alphas <- pick_alphas(p, M, params, model="StMAR", restricted=restricted, constraints=constraints)
+  all_regs <- lapply(1:M, function(i1) {
+    reg <- extractRegime(p, M, params, model="StMAR", restricted=restricted,
+                         constraints=constraints, regime=i1)
+    reg[-length(reg)]
+    })
+  reg_order <- c(regs_to_change[order(alphas[regs_to_change], decreasing=TRUE)], # GMAR type regimes
+    (1:M)[-regs_to_change][order(alphas[-regs_to_change], decreasing=TRUE)]) # StMAR type regimes
+  tmp_pars <- unlist(all_regs[reg_order])
+
+  if(!is.null(constraints) & any(reg_order != 1:M)) {
+    message(paste0("Order of the constraint matrices for was changed to ", toString(reg_order), "."))
+  }
+  if(restricted == TRUE) { # Add the missing AR parameters
+    q <- ifelse(is.null(constraints), p, ncol(constraints))
+    tmp_pars <- c(tmp_pars[seq(from=1, to=2*M, by=2)],
+                  params[(M + 1):(M + q)],
+                  tmp_pars[seq(from=2, to=2*M, by=2)])
+  }
+  if(length(regs_to_change) == M) {
+    new_dfs <- numeric(0)
+  } else {
+    new_dfs <- dfs[-regs_to_change][order(reg_order[(length(regs_to_change) + 1):M], decreasing=FALSE)]
+  }
+  pars <- c(tmp_pars, alphas[reg_order][-M], new_dfs)
+  return(list(params=pars, reg_order=reg_order, M=c(length(regs_to_change), M - length(regs_to_change))))
+}
+
+
 #' @title Pick phi0 or mean parameters from parameter vector
 #'
 #' @description \code{pick_phi0} picks and returns the phi0 or mean parameters from parameter vector.
@@ -66,13 +126,14 @@ pick_alphas <- function(p, M, params, model=c("GMAR", "StMAR", "G-StMAR"), restr
 }
 
 
-#' @title Pick \eqn{\phi_0}/\eqn{\mu}, AR-coefficients and variance parameters from parameter vector
+#' @title Pick \eqn{\phi_0} (or \eqn{\mu}), AR-coefficients and variance parameters from parameter vector
 #'
 #' @description \code{pick_pars} picks \eqn{\phi_0}/\eqn{\mu}, ar-coefficient and variance parameters from parameter vector
 #'
 #' @inheritParams loglikelihood_int
-#' @return Returns a \eqn{(Mx(p+2))} matrix containing the parameters, column for each component. First row for \eqn{\phi_0}/\eqn{\mu} depending on the parametrization,
-#'   second row for \eqn{\phi_1}, second last row for \eqn{\phi_p} and last row for \eqn{\sigma^2}.
+#' @return Returns a \eqn{(Mx(p+2))} matrix containing the parameters, column for each component.
+#'  First row for \eqn{\phi_0} or \eqn{\mu} depending on the parametrization,
+#'  second row for \eqn{\phi_1},..., second last row for \eqn{\phi_p} and last row for \eqn{\sigma^2}.
 
 pick_pars <- function(p, M, params, model=c("GMAR", "StMAR", "G-StMAR"), restricted=FALSE, constraints=NULL) {
   params <- removeAllConstraints(p=p, M=M, params=params, model=model, restricted=restricted, constraints=constraints)
@@ -130,49 +191,6 @@ change_parametrization <- function(p, M, params, model=c("GMAR", "StMAR", "G-StM
 }
 
 
-#' @title Calculate and return regime means \eqn{\mu_{m}}
-#'
-#' @description \code{get_regime_means} calculates regime means \eqn{\mu_{m} =  \phi_{m,0}/(1-\sum\phi_{i,m})}
-#'   for the given GMAR, StMAR or G-StMAR model
-#'
-#' @inheritParams simulateGSMAR
-#' @return Returns a length \code{M} vector containing regime mean \eqn{\mu_{m}} in the m:th column, \eqn{m=1,..,M}.
-#' @inherit isStationary references
-#' @examples
-#' # GMAR model
-#' params13 <- c(1.4, 0.88, 0.26, 2.46, 0.82, 0.74, 5.0, 0.68, 5.2, 0.72, 0.2)
-#' gmar13 <- GSMAR(data=VIX, p=1, M=3, params=params13, model="GMAR")
-#' get_regime_means(gmar13)
-#'
-#' # StMAR model
-#' params12t <- c(1.38, 0.88, 0.27, 3.8, 0.74, 3.15, 0.8, 100, 3.6)
-#' stmar12t <- GSMAR(data=VIX, p=1, M=2, params=params12t, model="StMAR")
-#' get_regime_means(stmar12t)
-#'
-#' # G-StMAR model (similar to the StMAR model above)
-#' params12gs <- c(1.38, 0.88, 0.27, 3.8, 0.74, 3.15, 0.8, 3.6)
-#' gstmar12 <- GSMAR(data=VIX, p=1, M=c(1, 1), params=params12gs,
-#'  model="G-StMAR")
-#' get_regime_means(gstmar12)
-#' @export
-
-get_regime_means <- function(gsmar) {
-  check_gsmar(gsmar)
-  p <- gsmar$model$p
-  M <- gsmar$model$M
-  params <- gsmar$params
-  model <- gsmar$model$model
-  restricted <- gsmar$model$restricted
-  constraints <- gsmar$model$constraints
-
-  if(gsmar$model$parametrization == "intercept") {
-    params <- change_parametrization(p=p, M=M, params=params, model=model, restricted=restricted,
-                                     constraints=constraints, change_to="mean")
-  }
-
-  pick_phi0(p=p, M=M, params=params, model=model, restricted=restricted, constraints=constraints)
-}
-
 
 #' @title Calculate absolute values of the roots of the AR characteristic polynomials
 #'
