@@ -15,7 +15,7 @@ format_valuef <- function(digits) {
 
 #' @describeIn GSMAR print method
 #' @inheritParams plot.gsmar
-#' @param digits number of digits to be printed
+#' @param digits number of digits to be printed (max 20)
 #' @param summary_print if set to \code{TRUE} then the print will include approximate
 #'  standard errors, log-likelihood and information criteria values. Supported only for
 #'  models with data.
@@ -73,7 +73,7 @@ print.gsmar <- function(x, ..., digits=2, summary_print=FALSE) {
     }
   }
 
-  cat("Model:\n", paste0("p = ", p, ", "))
+  cat("Model:\n", paste0(model, ", p = ", p, ", "))
   if(model == "G-StMAR") {
     cat(paste0("M1 = ", M[1], ", M2 = ", M[2], ", "))
   } else {
@@ -82,7 +82,7 @@ print.gsmar <- function(x, ..., digits=2, summary_print=FALSE) {
   cat(ifelse(gsmar$model$conditional, "conditional,", "exact,"),
       ifelse(gsmar$model$parametrization == "mean", "mean parametrization,", "intercept parametrization,"),
       ifelse(restricted, "AR parameters restricted,", "not restricted,"),
-      ifelse(is.null(constraints), "no constraints", "linear constraints employed"), "\n")
+      ifelse(is.null(constraints), "no constraints", "linear constraints imposed"), "\n")
 
   if(summary_print == TRUE) {
     IC <- gsmar$IC
@@ -107,9 +107,9 @@ print.gsmar <- function(x, ..., digits=2, summary_print=FALSE) {
       regime_type <- ifelse(m <= M1, "GMAR", "StMAR")
     }
     cat(paste("Regime", m))
-    if(model == "G-StMAR") cat(paste0(" (", regime_type, "-type)"))
+    if(model == "G-StMAR") cat(paste0(" (", regime_type, " type)"))
 
-    if(summary_print==TRUE) cat(paste("\nModulus of poly roots:", paste0(format_value(all_ar_roots[[m]]), collapse=", ")))
+    if(summary_print == TRUE) cat(paste("\nModulus of poly roots:", paste0(format_value(all_ar_roots[[m]]), collapse=", ")))
 
     cat(paste("\nMix weight:", format_value(alphas[m])))
     print_err(alphas_err[m])
@@ -133,33 +133,40 @@ print.gsmar <- function(x, ..., digits=2, summary_print=FALSE) {
     add_string(const_spaces=4, all_phi0[m], phi0_err[m])
 
     for(i1 in seq_len(p)) {
-      cat(paste0(" + [", format_value(pars[1 + i1, m]), "]Y.l", i1))
-      nspaces <- ifelse(i1 == 1, 3, 7)
+      cat(paste0(" + [", format_value(pars[1 + i1, m]), "]Y.", i1))
+      nspaces <- ifelse(i1 == 1, 3, 6)
       add_string(const_spaces=nspaces, pars[1 + i1, m], pars_err[1 + i1, m])
     }
     cat(" + ")
 
     if(regime_type == "GMAR") {
       cat(paste0("[sqrt(", format_value(pars[nrow(pars), m]), ")]eps"))
-      add_string(const_spaces=12, pars[nrow(pars), m], pars_err[nrow(pars_err), m])
+      add_string(const_spaces=11, pars[nrow(pars), m], pars_err[nrow(pars_err), m])
     } else {
       cat("[cond_sd]eps")
     }
     cat("\n")
-    if(summary_print == TRUE) cat(paste0(err_string, collapse=""))
-    cat("\n")
+    if(summary_print == TRUE) cat(paste0(err_string, collapse=""), '\n')
   }
+
+  if(summary_print == TRUE) {
+    um <- gsmar$uncond_moments
+    cat("\nProcess mean:", format_value(um$uncond_mean), "\n")
+    cat("Process var: ", format_value(um$uncond_var), "\n")
+    cat("First p autocors:", format_value(um$autocors), "\n")
+  }
+
   invisible(gsmar)
 }
 
 
 #' @title Print method from objects of class 'gsmarsum'
 #'
-#' @description \code{print.gsmarsum} is a print method for object \code{'gsmarsum'} generated
-#'   by \code{summary.gsmar()}. Approximate tandard errors are printed in brackets.
+#' @description \code{print.gsmarsum} is a print method for object \code{'gsmarsum'} created
+#'  with \code{summary.gsmar}. Approximate standard errors are printed in brackets.
 #'
-#' @param x object of class 'gsmarsum' generated by \code{summary.gsmar()}.
-#' @param ... currectly not used.
+#' @param x object of class 'gsmarsum' generated by \code{summary.gsmar}.
+#' @param ... currently not used.
 #' @param digits number of digits to be printed
 #' @examples
 #' \donttest{
@@ -171,7 +178,8 @@ print.gsmar <- function(x, ..., digits=2, summary_print=FALSE) {
 #' }
 #' @export
 
-print.gsmarsum <- function(x, ..., digits=2) {
+print.gsmarsum <- function(x, ..., digits) {
+  if(missing(digits)) digits <- x$digits
   print.gsmar(x$gsmar, digits=digits, summary_print=TRUE)
 }
 
@@ -179,18 +187,17 @@ print.gsmarsum <- function(x, ..., digits=2) {
 
 #' @title print method for class 'gsmarpred' objects
 #'
-#' @description \code{print.gsmarpred} is print method for object generated
-#'  by \code{predict.gsmar()}.
+#' @description \code{print.gsmarpred} is print method for object created
+#'  by \code{predict.gsmar}.
 #'
-#' @param x object of class \code{'gsmarpred'} generated by \code{predict.gsmar()}.
+#' @param x object of class \code{'gsmarpred'} generated by \code{predict.gsmar}.
 #' @param digits number of decimals to print
 #' @param ... currectly not used.
 #' @examples
 #'  \donttest{
 #'  # GMAR-model
-#'  params12 <- c(1.12321, 0.90557, 0.29288, 4.53131,
-#'                0.70309, 3.21154, 0.83888)
-#'  gmar12 <- GSMAR(VIX, 1, 2, params12)
+#'  params12 <-  c(0.18, 0.93, 0.01, 0.86, 0.68, 0.02, 0.88)
+#'  gmar12 <- GSMAR(logVIX, 1, 2, params12)
 #'  pred <- predict(gmar12, n_ahead=10, plotRes=FALSE)
 #'  pred
 #'  print(pred, digits=3)
@@ -237,7 +244,7 @@ print.gsmarpred <- function(x, ..., digits=2) {
 
 
 #' @describeIn quantileResidualTests print method for class 'qrtest'
-#' @param x object of class \code{'qrtest'} generated by the function \code{quantileResidualTests()}.
+#' @param x object of class \code{'qrtest'} created with the function \code{quantileResidualTests}.
 #' @param ... graphical parameters passed to \code{segments} in \code{plot.qrtest}.
 #'  Currectly not used in \code{print.qrtest}
 #' @param digits number of decimals to print
