@@ -17,8 +17,6 @@
 #'            \emph{The Econometrics Journal}, \strong{15}, 358-393.
 #'    \item Kalliovirta L., Meitz M. and Saikkonen P. 2015. Gaussian Mixture Autoregressive model for univariate time series.
 #'            \emph{Journal of Time Series Analysis}, \strong{36}, 247-266.
-#'    \item Lutkepohl H. 2005. New Introduction to Multiple Time Series Analysis.
-#'            \emph{Springer}.
 #'    \item Meitz M., Preve D., Saikkonen P. 2018. A mixture autoregressive model based on Student's t-distribution.
 #'            arXiv:1805.04010 \strong{[econ.EM]}.
 #'    \item There are currently no published references for G-StMAR model, but it's a straight forward generalization with
@@ -67,15 +65,11 @@ quantileResiduals_int <- function(data, p, M, params, model=c("GMAR", "StMAR", "
   # Calculate inverse Gamma_m and calculate the matrix products in mv normal and t-distribution (Galbraith and Galbraith 1974)
   matProd <- matrix(nrow=n_obs - p + 1, ncol = M)
   invG <- array(dim=c(p, p, M))
-  if(p == 1) { # Lutkepohl (2005), s.15-29
+  if(p == 1) { # Galbraith, R., Galbraith, J. (1974)
     for(i1 in 1:M) {
-      A <- pars[p + 1, i1]
-      Sigma <- as.matrix(sigmas[i1])
-      VecGamma <- solve(1 - kronecker(A, A), Sigma)
-      invG[, , i1] <- as.matrix(1/VecGamma)
-      matProd[,i1] <- (Y - mu[i1])*invG[, , i1]*(Y - mu[i1])
-    }
-  } else { # Galbraith, R., Galbraith, J., (1974)
+      invG[, , i1] <- (1 - pars[p + 1, i1]^2)/sigmas[i1]
+      matProd[, i1] <- (Y - mu[i1])*invG[, , i1]*(Y - mu[i1])  }
+  } else {
     for(i1 in 1:M) {
       ARcoefs <- pars[2:(p + 1), i1]
       U <- diag(1, nrow=p, ncol=p)
@@ -171,7 +165,12 @@ quantileResiduals_int <- function(data, p, M, params, model=c("GMAR", "StMAR", "
               alpha_mt[i2, i1]*exp(lgamma(0.5*(1 + dfs[i1] + p)) - lgamma(0.5*(dfs[i1] + p)))/sqrt(sigma_mt[i2, i1]*pi*(dfs[i1] + p - 2))*
                 (1 + ((y_t-mu_mt[i2, i1])^2)/((dfs[i1] + p - 2)*sigma_mt[i2, i1]))^(-0.5*(1 + dfs[i1] + p))
             }
-            res0[i2, i1] <- integrate(f_mt, lower=-Inf, upper=Y2[i2])$value # Integrate PDF numerically
+            res0[i2, i1] <- tryCatch(integrate(f_mt, lower=-Inf, upper=Y2[i2])$value, # Integrate PDF numerically
+                                     error=function(e) {
+                                       warning("Couldn't numerically integrate all quantile residuals.")
+                                       warning(e)
+                                       return(NA)
+                                     })
           }
         }
       }
@@ -183,7 +182,12 @@ quantileResiduals_int <- function(data, p, M, params, model=c("GMAR", "StMAR", "
             alpha_mt[i2, i1]*exp(lgamma(0.5*(1 + dfs[i1] + p)) - lgamma(0.5*(dfs[i1] + p)))/sqrt(sigma_mt[i2, i1]*pi*(dfs[i1] + p - 2))*
               (1 + ((y_t - mu_mt[i2, i1])^2)/((dfs[i1] + p - 2)*sigma_mt[i2, i1]))^(-0.5*(1 + dfs[i1] + p))
           }
-          res0[i2, i1] <- integrate(f_mt, lower=-Inf, upper=Y2[i2])$value # Integrate PDF numerically
+          res0[i2, i1] <- tryCatch(integrate(f_mt, lower=-Inf, upper=Y2[i2])$value, # Integrate PDF numerically
+                                   error=function(e) {
+                                     warning("Couldn't numerically integrate all quantile residuals.")
+                                     warning(e)
+                                     return(NA)
+                                   })
         }
       }
     }
@@ -223,7 +227,12 @@ quantileResiduals_int <- function(data, p, M, params, model=c("GMAR", "StMAR", "
               alpha_mt[i2, M1 + i1]*exp(lgamma(0.5*(1 + dfs[i1] + p)) - lgamma(0.5*(dfs[i1] + p)))/sqrt(sigma_mt[i2, i1]*pi*(dfs[i1] + p - 2))*
                 (1 + ((y_t - mu_mt[i2, M1 + i1])^2)/((dfs[i1] + p - 2)*sigma_mt[i2, i1]))^(-0.5*(1 + dfs[i1] + p))
             }
-            resM2[i2, i1] <- integrate(f_mt, lower=-Inf, upper=Y2[i2])$value # Integrate PDF numerically
+            resM2[i2, i1] <- tryCatch(integrate(f_mt, lower=-Inf, upper=Y2[i2])$value, # Integrate PDF numerically
+                                      error=function(e) {
+                                        warning("Couldn't numerically integrate all quantile residuals.")
+                                        warning(e)
+                                        return(NA)
+                                      })
           }
         }
       }
@@ -235,11 +244,16 @@ quantileResiduals_int <- function(data, p, M, params, model=c("GMAR", "StMAR", "
             alpha_mt[i2, M1 + i1]*exp(lgamma(0.5*(1 + dfs[i1] + p)) - lgamma(0.5*(dfs[i1] + p)))/sqrt(sigma_mt[i2, i1]*pi*(dfs[i1] + p - 2))*
               (1 + ((y_t - mu_mt[i2, M1 + i1])^2)/((dfs[i1] + p - 2)*sigma_mt[i2, i1]))^(-0.5*(1 + dfs[i1] + p))
           }
-          resM2[i2, i1] <- integrate(f_mt, lower=-Inf, upper=Y2[i2])$value # Integrate PDF numerically
+          resM2[i2, i1] <- tryCatch(integrate(f_mt, lower=-Inf, upper=Y2[i2])$value, # Integrate PDF numerically
+                                       error=function(e) {
+                                         warning("Couldn't numerically integrate all quantile residuals.")
+                                         warning(e)
+                                         return(NA)
+                                       })
         }
       }
     }
-    res0 = rowSums(cbind(resM1, resM2))
+    res0 <- rowSums(cbind(resM1, resM2))
   }
   # To prevent problems with numerical approximations
   res0[which(res0 >= 1)] <- 1 - 2e-16
@@ -260,39 +274,36 @@ quantileResiduals_int <- function(data, p, M, params, model=c("GMAR", "StMAR", "
 #'   Install the suggested package "gsl" for faster evaluation of the quantile residuals of StMAR and G-StMAR models.
 #' @examples
 #' # GMAR model
-#' params12 <- c(1.12, 0.9, 0.29, 4.53, 0.7, 3.22, 0.84)
-#' qr12 <- quantileResiduals(VIX, 1, 2, params12)
+#' params12 <- c(0.18, 0.93, 0.01, 0.86, 0.68, 0.02, 0.88)
+#' quantileResiduals(logVIX, 1, 2, params12)
 #'
-#' # Restricted GMAR model
-#' params12r <- c(1.4, 1.8, 0.88, 0.29, 3.18, 0.84)
-#' qr12r <- quantileResiduals(VIX, 1, 2, params12r, restricted=TRUE)
+#' # Restricted GMAR model, outside parameter space
+#' params12r <- c(0.21, 0.23, 0.92, 0.01, 0.02, 0.86)
+#' quantileResiduals(logVIX, 1, 2, params12r, restricted=TRUE)
 #'
-#' # StMAR model
-#' params12t <- c(1.1, 0.9, 0.3, 4.5, 0.7, 3.2, 0.8, 5, 8)
-#' qr12t <- quantileResiduals(VIX, 1, 2, params12t, model="StMAR")
-#'
-#' # Non-mixture version of StMAR model
-#' params11t <- c(0.76, 0.93, 1.35, 2.4)
-#' qr11t <- quantileResiduals(VIX, 1, 1, params11t, model="StMAR")
+#' # Non-mixture version of StMAR model, outside parameter space
+#' params11t <- c(0.16, 0.93, 0.01, 3.01)
+#' quantileResiduals(logVIX, 1, 1, params11t, model="StMAR")
 #'
 #' # G-StMAR model
-#' params12gs <- c(1.5, 0.8, 1.5, 2.9, 0.8, 1.1, 0.6, 3)
-#' qr12gs <- quantileResiduals(VIX, 1, c(1, 1), params12gs, model="G-StMAR")
+#' params12gs <- c(0.86, 0.68, 0.02, 0.18, 0.93, 0.01, 0.11, 44.36)
+#' quantileResiduals(logVIX, 1, c(1, 1), params12gs, model="G-StMAR")
 #'
 #' # Restricted G-StMAR model
-#' params13gsr <- c(1.3, 1, 1.4, 0.8, 0.4, 2, 0.2, 0.25, 0.15, 20)
-#' qr12gsr <- quantileResiduals(VIX, 1, c(2, 1), params13gsr, model="G-StMAR", restricted=TRUE)
+#' params12gsr <- c(0.31, 0.33, 0.88, 0.01, 0.02, 0.77, 2.72)
+#' quantileResiduals(logVIX, 1, c(1, 1), params12gsr, model="G-StMAR",
+#'  restricted=TRUE)
 #'
 #' # GMAR model as a mixture of AR(2) and AR(1) models
 #' constraints <- list(diag(1, ncol=2, nrow=2), as.matrix(c(1, 0)))
-#' params22c <- c(1.2, 0.85, 0.04, 0.3, 3.3, 0.77, 2.8, 0.77)
-#' qr22c <- quantileResiduals(VIX, 2, 2, params22c, constraints=constraints)
+#' params22c <- c(0.61, 0.83, -0.06, 0.02, 0.21, 0.91, 0.01, 0.16)
+#' quantileResiduals(logVIX, 2, 2, params22c, constraints=constraints)
 #'
 #' # Such StMAR(3,2) that the AR coefficients are restricted to be
 #' # the same for both regimes and that the second AR coefficients are
 #' # constrained to zero.
-#' params32trc <- c(2.2, 1.8, 0.88, -0.03, 2.4, 0.27, 0.40, 3.9, 1000)
-#' qr32trc <- quantileResiduals(VIX, 3, 2, params32trc, model="StMAR",
+#' params32trc <- c(0.35, 0.33, 0.88, -0.02, 0.01, 0.01, 0.36, 4.53, 1000)
+#' quantileResiduals(logVIX, 3, 2, params32trc, model="StMAR",
 #'  restricted=TRUE, constraints=matrix(c(1, 0, 0, 0, 0, 1), ncol=2))
 #' @export
 
